@@ -26,57 +26,6 @@
 #include "opkg_message.h"
 #include "xfuncs.h"
 
-struct errlist {
-    char *errmsg;
-    struct errlist *next;
-};
-
-static struct errlist *error_list_head, *error_list_tail;
-
-static void push_error_list(char *msg)
-{
-    struct errlist *e;
-
-    e = xcalloc(1, sizeof(struct errlist));
-    e->errmsg = xstrdup(msg);
-    e->next = NULL;
-
-    if (error_list_head) {
-        error_list_tail->next = e;
-        error_list_tail = e;
-    } else {
-        error_list_head = error_list_tail = e;
-    }
-}
-
-void free_error_list(void)
-{
-    struct errlist *err, *err_tmp;
-
-    err = error_list_head;
-    while (err != NULL) {
-        free(err->errmsg);
-        err_tmp = err;
-        err = err->next;
-        free(err_tmp);
-    }
-    error_list_head = NULL;
-}
-
-void print_error_list(void)
-{
-    struct errlist *err = error_list_head;
-
-    if (err) {
-        fprintf(stderr, "Collected errors:\n");
-        /* Here we print the errors collected and free the list */
-        while (err != NULL) {
-            fprintf(stderr, " * %s", err->errmsg);
-            err = err->next;
-        }
-    }
-}
-
 void opkg_message(message_level_t level, const char *fmt, ...)
 {
     va_list ap;
@@ -109,7 +58,7 @@ void opkg_message(message_level_t level, const char *fmt, ...)
         if (ret >= MSG_LEN) {
             fprintf(stderr, "%s: Message truncated.\n", __FUNCTION__);
         }
-        push_error_list(msg);
+        fprintf(stderr, "%s", msg);
     } else {
         int ret;
         ret = vprintf(fmt, ap);
