@@ -61,6 +61,7 @@ enum {
     ARGS_OPT_AUTOREMOVE,
     ARGS_OPT_VOLATILE_CACHE,
     ARGS_OPT_COMBINE,
+    ARGS_OPT_INSTALL_RECOMMENDS,
     ARGS_OPT_NO_INSTALL_RECOMMENDS,
     ARGS_OPT_CACHE_DIR,
     ARGS_OPT_HOST_CACHE_DIR,
@@ -108,6 +109,7 @@ static struct option long_options[] = {
     {"noaction", 0, 0, ARGS_OPT_NOACTION},
     {"download-only", 0, 0, ARGS_OPT_DOWNLOAD_ONLY},
     {"nodeps", 0, 0, ARGS_OPT_NODEPS},
+    {"install-recommends", 0, 0, ARGS_OPT_INSTALL_RECOMMENDS},
     {"no-install-recommends", 0, 0, ARGS_OPT_NO_INSTALL_RECOMMENDS},
     {"offline-root", 1, 0, 'o'},
     {"add-arch", 1, 0, ARGS_OPT_ADD_ARCH},
@@ -129,39 +131,42 @@ static struct option long_options[] = {
     {0, 0, 0, 0}
 };
 
-static int args_parse(int argc, char *argv[])
+const char *short_options = "Ad:f:no:p:l:t:vV::";
+
+static void store_str_arg(char **dest, const char *arg)
+{
+    free(*dest);
+    *dest = xstrdup(arg);
+}
+
+/*
+ * Performs stage 1 command line argument parsing.
+ *
+ * Stage 1 command line parsing handles all arguments that are required to parse
+ * the configuration files.
+ */
+static int args_parse_stage1(int argc, char *argv[])
 {
     int c;
     int option_index = 0;
     int parse_err = 0;
-    char *tuple, *targ;
     char *solver_version = NULL;
 
+    // Make sure that getopt starts from the beginning
+    optind = 1;
     while (1) {
-        c = getopt_long(argc, argv, "Ad:f:no:p:l:t:vV::", long_options,
+        c = getopt_long(argc, argv, short_options, long_options,
                              &option_index);
         if (c == -1)
             break;
 
         switch (c) {
-        case 'A':
-            opkg_config->query_all = 1;
-            break;
-        case 'd':
-            opkg_config->dest_str = xstrdup(optarg);
-            break;
         case 'f':
             opkg_config->conf_file = xstrdup(optarg);
             break;
         case 'o':
             opkg_config->offline_root = xstrdup(optarg);
             break;
-        case 't':
-            opkg_config->tmp_dir = xstrdup(optarg);
-            break;
-        case 'l':
-            opkg_config->lists_dir = xstrdup(optarg);
-            break;
         case 'v':
             solver_version = opkg_solver_version_alloc();
             if (solver_version) {
@@ -176,6 +181,62 @@ static int args_parse(int argc, char *argv[])
             if (optarg != NULL)
                 opkg_config->verbosity = atoi(optarg);
             break;
+        case '?':
+            parse_err = -1;
+            break;
+        default:
+            // All other arguments are handled in stage 2 command line parsing.
+            break;
+        }
+    }
+
+    if (parse_err)
+        return parse_err;
+    else
+        return optind;
+}
+
+/*
+ * Performs stage 2 command line argument parsing.
+ *
+ * Stage 2 command line parsing handles all other arguments. Stage 2 parsing
+ * is performed after the configuration files have been loaded thus allowing
+ * configuration options to be overridden from the command line.
+ */
+static int args_parse_stage2(int argc, char *argv[])
+{
+    int c;
+    int option_index = 0;
+    int parse_err = 0;
+    char *tuple, *targ;
+
+    // Make sure that getopt starts from the beginning
+    optind = 1;
+    while (1) {
+        c = getopt_long(argc, argv, short_options, long_options,
+                             &option_index);
+        if (c == -1)
+            break;
+
+        switch (c) {
+        case 'A':
+            opkg_config->query_all = 1;
+            break;
+        case 'd':
+            store_str_arg(&opkg_config->dest_str, optarg);
+            break;
+        case 'o':
+        case 'f':
+        case 'v':
+        case 'V':
+            // Already handled in stage 1 command line parsing
+            break;
+        case 't':
+            store_str_arg(&opkg_config->tmp_dir, optarg);
+            break;
+        case 'l':
+            store_str_arg(&opkg_config->lists_dir, optarg);
+            break;
         case ARGS_OPT_AUTOREMOVE:
             opkg_config->autoremove = 1;
             break;
@@ -249,7 +310,7 @@ static int args_parse(int argc, char *argv[])
             opkg_config->download_only = 1;
             break;
         case ARGS_OPT_CACHE_DIR:
-            opkg_config->cache_dir = xstrdup(optarg);
+            store_str_arg(&opkg_config->cache_dir, optarg);
             break;
         case ARGS_OPT_HOST_CACHE_DIR:
             opkg_config->host_cache_dir = 1;
@@ -257,6 +318,9 @@ static int args_parse(int argc, char *argv[])
         case ARGS_OPT_VOLATILE_CACHE:
             opkg_config->volatile_cache = 1;
             break;
+        case ARGS_OPT_INSTALL_RECOMMENDS:
+            opkg_config->no_install_recommends = 0;
+            break;
         case ARGS_OPT_NO_INSTALL_RECOMMENDS:
             opkg_config->no_install_recommends = 1;
             break;
@@ -264,19 +328,13 @@ static int args_parse(int argc, char *argv[])
            opkg_config->short_description = 1;
            break;
         case ARGS_OPT_FIELDS_FILTER:
-            opkg_config->fields_filter = xstrdup(optarg);
+            store_str_arg(&opkg_config->fields_filter, optarg);
             break;
         case ARGS_OPT_COMBINE:
             opkg_config->combine = 1;
             break;
-        case ':':
-            parse_err = -1;
-            break;
-        case '?':
-            parse_err = -1;
-            break;
         default:
-            printf("Confusion: getopt_long returned %d\n", c);
+            fprintf(stderr, "Encountered unhandled option %d during command line parsing\n", c);
         }
     }
 
@@ -377,7 +435,7 @@ static void usage()
     printf("\t--noaction                      No action -- test only\n");
     printf("\t--download-only                 No action -- download only\n");
     printf("\t--nodeps                        Do not follow dependencies\n");
-    printf("\t--no-install-recommends         Do not install any recommended packages\n");
+    printf("\t--[no-]install-recommends       (Do not) install any recommended packages\n");
     printf("\t--force-removal-of-dependent-packages\n");
     printf("\t                                Remove package and all dependencies\n");
     printf("\t--autoremove                    Remove packages that were installed\n");
@@ -412,7 +470,7 @@ int main(int argc, char *argv[])
     setlocale(LC_ALL, "");
     opkg_config->verbosity = NOTICE;
 
-    opts = args_parse(argc, argv);
+    opts = args_parse_stage1(argc, argv);
     if (opts == argc || opts < 0) {
         fprintf(stderr, "opkg must have one sub-command argument\n");
         usage();
@@ -452,7 +510,17 @@ int main(int argc, char *argv[])
     opkg_config->pfm = cmd->pfm;
 
     if (!noloadconf) {
-        if (opkg_conf_load())
+        if (opkg_conf_read())
+            goto err0;
+    }
+
+    if (args_parse_stage2(argc, argv) < 0) {
+        fprintf(stderr, "Stage 2 argument parsing failed.\n");
+        usage();
+    }
+
+    if (!noloadconf) {
+        if (opkg_conf_finalize())
             goto err0;
     }
 
